@@ -1,7 +1,7 @@
 package POE::Component::IKC::LocalKernel;
 
 ############################################################
-# $Id: LocalKernel.pm,v 1.6 2004/05/13 19:51:21 fil Exp $
+# $Id: LocalKernel.pm,v 1.7 2005/06/09 04:20:55 fil Exp $
 # Copyright 1999,2001,2002,2004 Philip Gwyn.  All rights reserved.
 # This program is free software; you can redistribute it and/or modify
 # it under the same terms as Perl itself.
@@ -101,6 +101,10 @@ sub sig_INT
 __DATA__
 
 $Log: LocalKernel.pm,v $
+Revision 1.7  2005/06/09 04:20:55  fil
+Reconciled
+Added check to put() to a closed wheel in Channel
+
 Revision 1.6  2004/05/13 19:51:21  fil
 Moved to signal_handled
 
